@@ -1,5 +1,6 @@
 package Netflow::Parser;
 
+use 5.006;
 use strict;
 use warnings;
 
@@ -12,19 +13,19 @@ use Carp;
 
 =head1 NAME
 
-Netflow::Parser - NetFlow datagram parser
+Netflow::Parser
 
 =head1 DESCRIPTION
 
-Netflow Parser supports currently L<NetFlow|https://en.wikipedia.org/wiki/NetFlow> V9 only
+A Netflow Parser supports only Netflow V9. The callback will be applied to each flow during packet parsing.
 
 =head1 VERSION
 
-Version 0.06.002
+Version 0.01
 
 =cut
 
-$Netflow::Parser::VERSION = '0.06.002';
+$Netflow::Parser::VERSION = '0.01';
 
 =head1 SYNOPSIS
 
@@ -36,22 +37,15 @@ $Netflow::Parser::VERSION = '0.06.002';
         );
 
     while(my $packet = take_packet_from_socket()) {
-        my $pp = $nfp->parse($packet);
+        my $content = $nfp->parse($packet);
 
-        # version, count, sysuptime, unix_secs, seqno and source_id
-        $pp->header;
-
-        # parsed flowsets
-        $pp->parsed;
-
-        # unparsed flowsets 
-        $pp->unparsed && persist_for_later($pp->unparsed);
+        $content->{unparsed_flowsets} && persist_for_later($content->{unparsed_flowsets});
     }
 
-    # persist templates if you want
     my @templates = $nfp->templates;
     foreach (@templates) {
         my ($id, $content) = each(%{$_});
+        $nfp->template($id); # persist template if you want
     }
 
 =head1 SUBROUTINES/METHODS
@@ -72,7 +66,7 @@ C<templates_data>
 
 C<flow_cb>
 
-callback method will be applied to each parsed flow
+callback method will be applied to each flow
 
 =item
 
@@ -131,10 +125,9 @@ return {
 
 sub parse {
     my ($self, $packet) = @_;
-
-    #my ($version) = unpack("n", $packet);
+    my ($version) = unpack("n", $packet);
     return $self->_parse_v9($packet);
-} ## end sub parse
+}
 
 =head2 templates()
 
@@ -200,20 +193,23 @@ sub _parse_v9 {
 
     eval { $version == 9 } || Carp::croak("the version of packet is not v9");
 
-    my $pp = Netflow::Parser::Packet->new(
-        'flowsets' => scalar(@flowsets),
-        'header'   => {
+    my $content = {
+        'templates'         => [],
+        'flows'             => [],
+        'unparsed_flowsets' => [],
+        'flowsets'          => scalar(@flowsets),
+        'header'            => {
             'version'   => $version,
             'count'     => $count,
             'sysuptime' => $sysuptime,
             'unix_secs' => $unix_secs,
             'seqno'     => $seqno,
             'source_id' => $source_id,
-        }
-    );
+        },
+    };
 
     scalar(@flowsets) > length($packet)
-        && warn sprintf("estimated %d flowsets > paket length %d",
+        && warn sprintf("extimated %d flowsets > paket length %d",
         scalar(@flowsets), length($packet));
     for (my $i = 0; $i < scalar(@flowsets); $i += 2) {
         my $flowset_id = $flowsets[$i];
@@ -222,12 +218,13 @@ sub _parse_v9 {
         my $flowset = substr($flowsets[$i + 1], 4);
         if ($flowset_id == 0) {
             if ($flowset) {
-                my @tmpl = $self->_parse_template_v9($flowset);
+                my @tmpl = $self->parse_template_v9($flowset);
                 if (@tmpl) {
-                    $pp->add_template(@tmpl);
+                    push @{ $content->{'templates'} }, @tmpl;
                 }
                 else {
-                    $pp->add_unparsed({ $flowset_id => $flowset });
+                    push @{ $content->{'unparsed_flowsets'} },
+                        { $flowset_id => $flowset };
                 }
             } ## end if ($flowset)
         } ## end if ($flowset_id == 0)
@@ -236,15 +233,16 @@ sub _parse_v9 {
             # 1 - Options Template FlowSet
             $self->{'verbose'} && $self->_debug("do nothing for flowset id 1");
 
-            $pp->add_unparsed({ $flowset_id => $flowset });
+            push @{ $content->{'unparsed_flowsets'} }, $flowset;
         } ## end elsif ($flowset_id == 1)
         elsif ($flowset_id > 255) {
             my @flows = $self->_parse_flowset_v9($flowset_id, $flowset);
             if (scalar(@flows)) {
-                $pp->add_parsed({ $flowset_id => [@flows] });
+                push @{ $content->{'flows'} }, { $flowset_id => [@flows] };
             }
             else {
-                $pp->add_unparsed({ $flowset_id => $flowset });
+                push @{ $content->{'unparsed_flowsets'} },
+                    { $flowset_id => $flowset };
             }
         } ## end elsif ($flowset_id > 255)
         else {
@@ -253,8 +251,7 @@ sub _parse_v9 {
                 && $self->_debug("Unknown FlowSet ID $flowset_id found");
         }
     } ## end for (my $i = 0; $i < scalar...)
-
-    return $pp;
+    return $content;
 } ## end sub _parse_v9
 
 #=head2 _parse_flowset_v9 ($flowset_id, $flowset)
@@ -309,7 +306,8 @@ sub _parse_flowset_v9 {
 
         $cb && $cb->($flow);
 
-        push @flows, $flow;
+        #        push @flows, $flow;
+        push @flows, $cb ? $cb->($flow) : $flow;
     } ## end while (($ofs + $tmpl_length...))
 
     return @flows;
@@ -371,24 +369,9 @@ sub _debug {
     print join(' ', "LINE[$line]:", @msg, $/);
 }
 
-=head1 EXAMPLE - L<Netflow Collector|https://metacpan.org/pod/Netflow::Collector> 
-
-my $p = Netflow::Parser->new(
-    verbose      => 1,
-    flow_cb      => sub {
-        my ($hr) = @_;
-        ...
-    }
-
-Netflow::Collector->new(
-    port => $port,
-    dispatch => sub {
-        $p->parse(@_) 
-    })->run();
-
 =head1 AUTHOR
 
-Alexei Pastuchov E<lt>palik at cpan dot orgE<gt>.
+Alexei Pastuchov E<lt>palik at cpan.orgE<gt>.
 
 =head1 REPOSITORY
 
@@ -396,7 +379,7 @@ L<https://github.com/p-alik/Netflow-Parser>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2014-2016 by Alexei Pastuchov E<lt>palik at cpan dot orgE<gt>.
+Copyright 2014 by Alexei Pastuchov E<lt>palik at cpan.orgE<gt>.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -404,104 +387,3 @@ it under the same terms as Perl itself.
 =cut
 
 1;    # End of Netflow::Parser
-
-{
-
-    package Netflow::Parser::Packet;
-    use strict;
-    use warnings;
-
-    use fields qw/
-        flowsets
-        header
-        templates
-        parsed
-        unparsed
-        /;
-
-    {
-        no strict 'refs';
-        foreach my $k (keys %{'Netflow::Parser::Packet::FIELDS'}) {
-            *{$k} = sub { shift->{$k} }
-        }
-    };
-
-    sub new {
-        my Netflow::Parser::Packet $self = shift;
-        my (%opts) = @_;
-        unless (ref $self) {
-            $self = fields::new($self);
-        }
-
-        $self->{header}
-            = Netflow::Parser::Packet::Header->new(%{ delete $opts{header} });
-
-        $self->{flowsets} = delete $opts{flowsets};
-
-        foreach (qw/templates parsed unparsed/) {
-            $self->{$_} = [];
-        }
-
-        return $self;
-    } ## end sub new
-
-    sub add_template {
-        my ($self, $tmpl) = @_;
-        push @{ $self->{templates} }, $tmpl;
-    }
-
-    sub add_parsed {
-        my ($self, $hr) = @_;
-        push @{ $self->{parsed} }, $hr;
-    }
-
-    sub add_unparsed {
-        my ($self, $hr) = @_;
-        push @{ $self->{unparsed} }, $hr;
-    }
-
-}
-
-1;    # End of Netflow::Parser::Packet
-
-{
-
-    package Netflow::Parser::Packet::Header;
-    use strict;
-    use warnings;
-
-    use fields qw/
-        version
-        count
-        sysuptime
-        unix_secs
-        seqno
-        source_id
-        /;
-
-    my @fields;
-    {
-        no strict 'refs';
-        @fields = keys %{'Netflow::Parser::Packet::Header::FIELDS'};
-        foreach (@fields) {
-            *{$_} = sub { shift->{$_} }
-        }
-    };
-
-    sub new {
-        my Netflow::Parser::Packet::Header $self = shift;
-        my (%opts) = @_;
-        unless (ref $self) {
-            $self = fields::new($self);
-        }
-
-        foreach (@fields) {
-            $opts{$_} || next;
-            $self->{$_} = delete $opts{$_};
-        }
-
-        return $self;
-    } ## end sub new
-}
-
-1;    # End of Netflow::Parser::Packet::Header
